@@ -31,8 +31,8 @@ namespace Botan {
 class RSA_Public_Data final {
    public:
       RSA_Public_Data(BigInt&& n, BigInt&& e) :
-            m_n(n),
-            m_e(e),
+            m_n(std::move(n)),
+            m_e(std::move(e)),
             m_monty_n(std::make_shared<Montgomery_Params>(m_n)),
             m_public_modulus_bits(m_n.bits()),
             m_public_modulus_bytes(m_n.bytes()) {}
@@ -62,12 +62,12 @@ class RSA_Public_Data final {
 class RSA_Private_Data final {
    public:
       RSA_Private_Data(BigInt&& d, BigInt&& p, BigInt&& q, BigInt&& d1, BigInt&& d2, BigInt&& c) :
-            m_d(d),
-            m_p(p),
-            m_q(q),
-            m_d1(d1),
-            m_d2(d2),
-            m_c(c),
+            m_d(std::move(d)),
+            m_p(std::move(p)),
+            m_q(std::move(q)),
+            m_d1(std::move(d1)),
+            m_d2(std::move(d2)),
+            m_c(std::move(c)),
             m_mod_p(m_p),
             m_mod_q(m_q),
             m_monty_p(std::make_shared<Montgomery_Params>(m_p, m_mod_p)),
@@ -178,6 +178,10 @@ AlgorithmIdentifier RSA_PublicKey::algorithm_identifier() const {
    return AlgorithmIdentifier(object_identifier(), AlgorithmIdentifier::USE_NULL_PARAM);
 }
 
+std::vector<uint8_t> RSA_PublicKey::raw_public_key_bits() const {
+   throw Not_Implemented("an RSA public key does not provide a raw binary representation.");
+}
+
 std::vector<uint8_t> RSA_PublicKey::public_key_bits() const {
    std::vector<uint8_t> output;
    DER_Encoder der(output);
@@ -463,13 +467,7 @@ class RSA_Private_Operation {
          const BigInt recovered = m_blinder.unblind(rsa_private_op(m_blinder.blind(input_bn)));
          BOTAN_ASSERT(input_bn == m_public->public_op(recovered), "RSA consistency check");
          BOTAN_ASSERT(m_public->public_modulus_bytes() == out.size(), "output size check");
-         BigInt::encode_1363(out, recovered);
-      }
-
-      secure_vector<uint8_t> raw_op(const uint8_t input[], size_t input_len) {
-         secure_vector<uint8_t> out(m_public->public_modulus_bytes());
-         raw_op(out, {input, input_len});
-         return out;
+         recovered.serialize_to(out);
       }
 
    private:
@@ -548,13 +546,16 @@ class RSA_Private_Operation {
 class RSA_Signature_Operation final : public PK_Ops::Signature,
                                       private RSA_Private_Operation {
    public:
-      void update(const uint8_t msg[], size_t msg_len) override { m_emsa->update(msg, msg_len); }
+      void update(std::span<const uint8_t> msg) override { m_emsa->update(msg.data(), msg.size()); }
 
-      secure_vector<uint8_t> sign(RandomNumberGenerator& rng) override {
+      std::vector<uint8_t> sign(RandomNumberGenerator& rng) override {
          const size_t max_input_bits = public_modulus_bits() - 1;
          const auto msg = m_emsa->raw_data();
          const auto padded = m_emsa->encoding_of(msg, max_input_bits, rng);
-         return raw_op(padded.data(), padded.size());
+
+         std::vector<uint8_t> out(public_modulus_bytes());
+         raw_op(out, padded);
+         return out;
       }
 
       size_t signature_length() const override { return public_modulus_bytes(); }
@@ -595,8 +596,10 @@ class RSA_Decryption_Operation final : public PK_Ops::Decryption_with_EME,
 
       size_t plaintext_length(size_t /*ctext_len*/) const override { return public_modulus_bytes(); }
 
-      secure_vector<uint8_t> raw_decrypt(const uint8_t input[], size_t input_len) override {
-         return raw_op(input, input_len);
+      secure_vector<uint8_t> raw_decrypt(std::span<const uint8_t> input) override {
+         secure_vector<uint8_t> out(public_modulus_bytes());
+         raw_op(out, input);
+         return out;
       }
 };
 
@@ -651,22 +654,20 @@ class RSA_Encryption_Operation final : public PK_Ops::Encryption_with_EME,
 
       size_t max_ptext_input_bits() const override { return public_modulus_bits() - 1; }
 
-      secure_vector<uint8_t> raw_encrypt(const uint8_t input[],
-                                         size_t input_len,
-                                         RandomNumberGenerator& /*rng*/) override {
-         BigInt input_bn(input, input_len);
-         return BigInt::encode_1363(public_op(input_bn), public_modulus_bytes());
+      std::vector<uint8_t> raw_encrypt(std::span<const uint8_t> input, RandomNumberGenerator& /*rng*/) override {
+         BigInt input_bn(input);
+         return public_op(input_bn).serialize(public_modulus_bytes());
       }
 };
 
 class RSA_Verify_Operation final : public PK_Ops::Verification,
                                    private RSA_Public_Operation {
    public:
-      void update(const uint8_t msg[], size_t msg_len) override { m_emsa->update(msg, msg_len); }
+      void update(std::span<const uint8_t> msg) override { m_emsa->update(msg.data(), msg.size()); }
 
-      bool is_valid_signature(const uint8_t sig[], size_t sig_len) override {
+      bool is_valid_signature(std::span<const uint8_t> sig) override {
          const auto msg = m_emsa->raw_data();
-         const auto message_repr = recover_message_repr(sig, sig_len);
+         const auto message_repr = recover_message_repr(sig.data(), sig.size());
          return m_emsa->verify(message_repr, msg, public_modulus_bits() - 1);
       }
 
@@ -681,7 +682,7 @@ class RSA_Verify_Operation final : public PK_Ops::Verification,
             throw Decoding_Error("RSA signature too large to be valid for this key");
          }
          BigInt input_bn(input, input_len);
-         return BigInt::encode(public_op(input_bn));
+         return public_op(input_bn).serialize();
       }
 
       std::unique_ptr<EMSA> m_emsa;
@@ -704,8 +705,8 @@ class RSA_KEM_Encryption_Operation final : public PK_Ops::KEM_Encryption_with_KD
          const BigInt r = BigInt::random_integer(rng, 1, get_n());
          const BigInt c = public_op(r);
 
-         BigInt::encode_1363(out_encapsulated_key, c);
-         BigInt::encode_1363(raw_shared_key, r);
+         c.serialize_to(out_encapsulated_key);
+         r.serialize_to(raw_shared_key);
       }
 };
 
