@@ -9,6 +9,7 @@
 #define BOTAN_TESTS_H_
 
 #include <botan/hex.h>
+#include <botan/mem_ops.h>
 #include <botan/rng.h>
 #include <botan/symkey.h>
 #include <botan/types.h>
@@ -17,6 +18,7 @@
 #include <map>
 #include <memory>
 #include <optional>
+#include <ranges>
 #include <set>
 #include <sstream>
 #include <string>
@@ -64,6 +66,11 @@ class Test_Options {
                    const std::string& data_dir,
                    const std::string& pkcs11_lib,
                    const std::string& provider,
+                   const std::string& tpm2_tcti_name,
+                   const std::string& tpm2_tcti_conf,
+                   size_t tpm2_persistent_rsa_handle,
+                   size_t tpm2_persistent_ecc_handle,
+                   const std::string& tpm2_persistent_auth_value,
                    const std::string& drbg_seed,
                    const std::string& xml_results_dir,
                    const std::vector<std::string>& report_properties,
@@ -74,12 +81,18 @@ class Test_Options {
                    bool run_online_tests,
                    bool run_long_tests,
                    bool run_memory_intensive_tests,
-                   bool abort_on_first_fail) :
+                   bool abort_on_first_fail,
+                   bool no_stdout) :
             m_requested_tests(requested_tests),
             m_skip_tests(skip_tests.begin(), skip_tests.end()),
             m_data_dir(data_dir),
             m_pkcs11_lib(pkcs11_lib),
             m_provider(provider),
+            m_tpm2_tcti_name(tpm2_tcti_name),
+            m_tpm2_tcti_conf(tpm2_tcti_conf),
+            m_tpm2_persistent_rsa_handle(tpm2_persistent_rsa_handle),
+            m_tpm2_persistent_ecc_handle(tpm2_persistent_ecc_handle),
+            m_tpm2_persistent_auth_value(tpm2_persistent_auth_value),
             m_drbg_seed(drbg_seed),
             m_xml_results_dir(xml_results_dir),
             m_report_properties(report_properties),
@@ -90,7 +103,8 @@ class Test_Options {
             m_run_online_tests(run_online_tests),
             m_run_long_tests(run_long_tests),
             m_run_memory_intensive_tests(run_memory_intensive_tests),
-            m_abort_on_first_fail(abort_on_first_fail) {}
+            m_abort_on_first_fail(abort_on_first_fail),
+            m_no_stdout(no_stdout) {}
 
       const std::vector<std::string>& requested_tests() const { return m_requested_tests; }
 
@@ -102,6 +116,20 @@ class Test_Options {
 
       const std::string& provider() const { return m_provider; }
 
+      const std::optional<std::string>& tpm2_tcti_name() const { return m_tpm2_tcti_name; }
+
+      const std::optional<std::string>& tpm2_tcti_conf() const { return m_tpm2_tcti_conf; }
+
+      uint32_t tpm2_persistent_rsa_handle() const { return static_cast<uint32_t>(m_tpm2_persistent_rsa_handle); }
+
+      uint32_t tpm2_persistent_ecc_handle() const { return static_cast<uint32_t>(m_tpm2_persistent_ecc_handle); }
+
+      std::vector<uint8_t> tpm2_persistent_auth_value() const {
+         std::span<const uint8_t> auth_value(Botan::cast_char_ptr_to_uint8(m_tpm2_persistent_auth_value.data()),
+                                             m_tpm2_persistent_auth_value.size());
+         return std::vector<uint8_t>(auth_value.begin(), auth_value.end());
+      }
+
       const std::string& drbg_seed() const { return m_drbg_seed; }
 
       const std::string& xml_results_dir() const { return m_xml_results_dir; }
@@ -122,6 +150,8 @@ class Test_Options {
 
       bool abort_on_first_fail() const { return m_abort_on_first_fail; }
 
+      bool no_stdout() const { return m_no_stdout; }
+
       bool verbose() const { return m_verbose; }
 
    private:
@@ -130,6 +160,11 @@ class Test_Options {
       std::string m_data_dir;
       std::string m_pkcs11_lib;
       std::string m_provider;
+      std::optional<std::string> m_tpm2_tcti_name;
+      std::optional<std::string> m_tpm2_tcti_conf;
+      size_t m_tpm2_persistent_rsa_handle;
+      size_t m_tpm2_persistent_ecc_handle;
+      std::string m_tpm2_persistent_auth_value;
       std::string m_drbg_seed;
       std::string m_xml_results_dir;
       std::vector<std::string> m_report_properties;
@@ -141,6 +176,7 @@ class Test_Options {
       bool m_run_long_tests;
       bool m_run_memory_intensive_tests;
       bool m_abort_on_first_fail;
+      bool m_no_stdout;
 };
 
 namespace detail {
@@ -558,11 +594,11 @@ class Test {
 
       virtual std::vector<std::string> possible_providers(const std::string&);
 
-      void set_test_name(const std::string& name) { m_test_name = name; }
+      void initialize(std::string test_name, CodeLocation location);
 
       const std::string& test_name() const { return m_test_name; }
 
-      void set_registration_location(CodeLocation location) { m_registration_location = std::move(location); }
+      Botan::RandomNumberGenerator& rng() const;
 
       const std::optional<CodeLocation>& registration_location() const { return m_registration_location; }
 
@@ -581,6 +617,8 @@ class Test {
       static std::unique_ptr<Test> get_test(const std::string& test_name);
       static bool test_needs_serialization(const std::string& test_name);
 
+      static std::string data_dir(const std::string& subdir);
+      static std::vector<std::string> files_in_data_dir(const std::string& subdir);
       static std::string data_file(const std::string& what);
       static std::string data_file_as_temporary_copy(const std::string& what);
 
@@ -592,10 +630,9 @@ class Test {
 
       template <typename Alloc>
       static std::vector<uint8_t, Alloc> mutate_vec(const std::vector<uint8_t, Alloc>& v,
+                                                    Botan::RandomNumberGenerator& rng,
                                                     bool maybe_resize = false,
                                                     size_t min_offset = 0) {
-         auto& rng = Test::rng();
-
          std::vector<uint8_t, Alloc> r = v;
 
          if(maybe_resize && (r.empty() || rng.next_byte() < 32)) {
@@ -616,7 +653,7 @@ class Test {
 
       static void set_test_options(const Test_Options& opts);
 
-      static void set_test_rng(std::shared_ptr<Botan::RandomNumberGenerator> rng);
+      static void set_test_rng_seed(std::span<const uint8_t> seed, size_t epoch = 0);
 
       static const Test_Options& options() { return m_opts; }
 
@@ -624,8 +661,6 @@ class Test {
 
       static bool run_memory_intensive_tests() { return options().run_memory_intensive_tests(); }
 
-      static const std::string& data_dir() { return options().data_dir(); }
-
       static const std::string& pkcs11_lib() { return options().pkcs11_lib(); }
 
       static std::string temp_file_name(const std::string& basename);
@@ -636,19 +671,24 @@ class Test {
       static std::string read_data_file(const std::string& path);
       static std::vector<uint8_t> read_binary_data_file(const std::string& path);
 
-      static Botan::RandomNumberGenerator& rng();
-      static std::shared_ptr<Botan::RandomNumberGenerator> rng_as_shared();
-      static std::string random_password();
+      static std::unique_ptr<Botan::RandomNumberGenerator> new_rng(std::string_view test_name);
+      static std::shared_ptr<Botan::RandomNumberGenerator> new_shared_rng(std::string_view test_name);
+
+      static std::string random_password(Botan::RandomNumberGenerator& rng);
       static uint64_t timestamp();  // nanoseconds arbitrary epoch
 
       static std::vector<Test::Result> flatten_result_lists(std::vector<std::vector<Test::Result>> result_lists);
 
    private:
       static Test_Options m_opts;
-      static std::shared_ptr<Botan::RandomNumberGenerator> m_test_rng;
+      static std::string m_test_rng_seed;
 
-      std::string m_test_name;                              // The string ID that was used to register this test
-      std::optional<CodeLocation> m_registration_location;  /// The source file location where the test was registered
+      /// The string ID that was used to register this test
+      std::string m_test_name;
+      /// The source file location where the test was registered
+      std::optional<CodeLocation> m_registration_location;
+      /// The test-specific RNG state
+      mutable std::unique_ptr<Botan::RandomNumberGenerator> m_test_rng;
 };
 
 /*
@@ -664,8 +704,7 @@ class TestClassRegistration {
                             CodeLocation registration_location) {
          Test::register_test(category, name, smoke_test, needs_serialization, [=] {
             auto test = std::make_unique<Test_Class>();
-            test->set_test_name(name);
-            test->set_registration_location(registration_location);
+            test->initialize(name, registration_location);
             return test;
          });
       }
@@ -742,21 +781,24 @@ class TestFnRegistration {
                          TestFns... fn) {
          Test::register_test(category, name, smoke_test, needs_serialization, [=] {
             auto test = std::make_unique<FnTest>(fn...);
-            test->set_test_name(name);
-            test->set_registration_location(std::move(registration_location));
+            test->initialize(name, registration_location);
             return test;
          });
       }
 };
 
+#define BOTAN_REGISTER_TEST_FN_IMPL(category, name, smoke_test, needs_serialization, fn0, ...) \
+   static const TestFnRegistration register_##fn0(                                             \
+      category, name, smoke_test, needs_serialization, {__FILE__, __LINE__}, fn0 __VA_OPT__(, ) __VA_ARGS__)
+
 #define BOTAN_REGISTER_TEST_FN(category, name, ...) \
-   static const TestFnRegistration reg_##fn_name(category, name, false, false, {__FILE__, __LINE__}, __VA_ARGS__)
+   BOTAN_REGISTER_TEST_FN_IMPL(category, name, false, false, __VA_ARGS__)
 #define BOTAN_REGISTER_SMOKE_TEST_FN(category, name, ...) \
-   static const TestFnRegistration reg_##fn_name(category, name, true, false, {__FILE__, __LINE__}, __VA_ARGS__)
+   BOTAN_REGISTER_TEST_FN_IMPL(category, name, true, false, __VA_ARGS__)
 #define BOTAN_REGISTER_SERIALIZED_TEST_FN(category, name, ...) \
-   static const TestFnRegistration reg_##fn_name(category, name, false, true {__FILE__, __LINE__}, __VA_ARGS__)
+   BOTAN_REGISTER_TEST_FN_IMPL(category, name, false, true, __VA_ARGS__)
 #define BOTAN_REGISTER_SERIALIZED_SMOKE_TEST_FN(category, name, ...) \
-   static const TestFnRegistration reg_##fn_name(category, name, true, true {__FILE__, __LINE__}, __VA_ARGS__)
+   BOTAN_REGISTER_TEST_FN_IMPL(category, name, true, true, __VA_ARGS__)
 
 class VarMap {
    public:
